@@ -1,89 +1,11 @@
-######################
 #######################
-classif.glm2boost=function(group,fdataobj,family=binomial(),basis.x=NULL,
-basis.b=NULL,CV=FALSE,...){
-C<-match.call()
-a<-list()
-mf <- match.call(expand.dots = FALSE)
-m <- match(c("group","fdataobj","family","basis.x","basis.b","CV"), names(mf), 0L)
-   if (is.fdata(fdataobj))   {
-       dataf<-data.frame("y"=group)
-       ldata<-list("df"=dataf,"X"=fdataobj)
-       formula<-formula(y~X)       
-       }
-   else {
-        if (is.matrix(fdataobj)) nms<-colnames(fdataobj)
-        else nms<-names(fdataobj)        
-#quita [ o $ del names
-#for (i in 1:length(nms)) nms[i]<-unlist(strsplit(nms[i], "[$]"))[[1]]
-#for (i in 1:length(nms)) nms[i]<-unlist(strsplit(nms[i], "[[]"))[[1]]
-        dataf<-data.frame("y"=group,fdataobj)
-        ldata<-list("df"=dataf)
-        aaa<-paste(nms,collapse="+")
-        formula<-formula(paste("y~",aaa,sep=""))      
-     }
-   newy<-y<-ldata$df$y
-   if (!is.factor(y)) y<-as.factor(y)
-   n<-length(y)
-   newdata<-ldata
-   ngroup<-nlevels(y)
-   ny<-levels(y)
-   prob<-rep(NA,ngroup)
-       if (!is.null(basis.x)) basis.x=list("X"=basis.x)
-       if (!is.null(basis.b)) basis.b=list("X"=basis.b)  
-   #nlevels(y) cambiar todos
-if (ngroup==2) {
-#      ny<-as.numeric(names(table(y)))
-      newy<-ifelse(y==ny[1],0,1)
-      newdata$df$y<-newy
-      a[[1]]<-fregre.glm(formula,data=newdata,family=binomial,basis.x=basis.x,
-              basis.b=basis.b,CV=CV)                
-      yest<-ifelse(a[[1]]$fitted.values<.5,ny[1],ny[2])
-      tab<-table(yest,y)
-      prob[1]=tab[1,1]/sum(tab[,1])
-      dtab<-dim(tab)
-      if (dtab[1]==dtab[2])    {
-           prob[2]=tab[2,2]/sum(tab[,2])
-           names(prob)<-ny
-         }
-      else prob[2]<-0
-      prob.group<-a$fitted.values
-      yest<-factor(yest,levels=ny)
-
-   }
-else {
-#   ny<-as.numeric(names(table(y)))
-   prob.group<-array(NA,dim=c(n,ngroup))
-   colnames(prob.group)<-ny
-   for (i in 1:ngroup) {
-              newy<-ifelse(y==ny[i],0,1)
-              newdata$df$y<-newy
-              a[[i]]<-fregre.glm(formula,data=newdata,family=family,basis.x=basis.x,
-              basis.b=basis.b,CV=CV)
-              prob.group[,i]<-a[[i]]$fitted.values
-              }
-   yest<-ny[apply(prob.group,1,which.min)]
-   yest<-factor(yest,levels=ny)
-   tab<-table(yest,y)
-   for (ii in 1:ngroup) {prob[ii]=tab[ii,ii]/sum(tab[,ii])}
-   names(prob)<-ny
-}
-max.prob=sum(diag(tab))/sum(tab)
-output<-list(fdataobj=fdataobj,group=y,group.est=as.factor(yest),
-prob.classification=prob,prob.group=prob.group,C=C,m=m,max.prob=max.prob,fit=a
-,formula=formula,data=newdata)
-class(output)="classif"
-return(output)
-}
-#############################
-classif.gsam2boost=function(group,fdataobj,family=binomial,weights=NULL,
+classif.gsam2boost=function(group,fdataobj,family=binomial(),weights=NULL,
 basis.x=NULL,basis.b=NULL,par.gsam=NULL,CV=FALSE,...){
 C<-match.call()
 a<-list()
 mf <- match.call(expand.dots = FALSE)
 m <- match(c( "group","fdataobj","family","basis.x","basis.b","par.gsam","CV"), names(mf), 0L)
 numg=nlevels(as.factor(group)) 
-
   if (is.fdata(fdataobj))   {
      gsam<-C[[3]];
      if (is.null(par.gsam$func)&is.null(par.gsam$k))   {gsam<-paste("s(X)",sep="")}
@@ -125,59 +47,144 @@ n<-length(y);
 newdata<-ldata
 
    ngroup<-nlevels(y)
-   ny<-levels(y)
+   lev<-levels(y)
 prob<-ngroup<-length(table(y))
 if (!is.null(basis.x)) basis.x=list("X"=basis.x)
 if (!is.null(basis.b)) basis.b=list("X"=basis.b)
 formula<-pf2
+lev <-levels(y)
 if (ngroup==2) {
-      ny<-as.numeric(names(table(y)))
-      newy<-ifelse(y==ny[1],0,1)
+      #lev<-(as.numeric(names(table(y)))
+      newy<-ifelse(y==lev[1],0,1)
       newdata$df$y<-newy
 #      formula<-formula(paste("y~",gsam),sep="")
       a[[1]]<-fregre.gsam(formula,data=newdata,family=family,weights=weights,basis.x=basis.x,
       basis.b=basis.b,CV=CV,...)
-      yest<-ifelse(a[[1]]$fitted.values<.5,ny[1],ny[2])
-      yest<-factor(yest,levels=ny)
+      yest<-ifelse(a[[1]]$fitted.values<.5,lev[1],lev[2])
+      yest<-factor(yest,levels=lev)
       tab<-table(yest,y)
       prob[1]=tab[1,1]/sum(tab[,1])
       dtab<-dim(tab)
       if (dtab[1]==dtab[2])    {
            prob[2]=tab[2,2]/sum(tab[,2])
-           names(prob)<-ny
+           names(prob)<-lev
          }
       else prob[2]<-0
-      prob.group<-a$fitted.values
+      prob.group<-a[[1]]$fitted.values
       #devolver a mayores y estimada
    }
 else {
-   ny<-as.numeric(names(table(y)))
+   #lev<-as.numeric(names(table(y)))
    prob.group<-array(NA,dim=c(n,ngroup))
-   colnames(prob.group)<-ny
+   colnames(prob.group)<-lev
    for (i in 1:ngroup) {
-              newy<-ifelse(y==ny[i],0,1)
+    # print(0)     
+              newy<-ifelse(y==lev[i],0,1)
               newdata$df$y<-newy
 #              formula<-formula(paste("y~",gsam),sep="")
-              a[[i]]<-fregre.gsam(formula,data=newdata,family=family,
-              weights=weights,basis.x=basis.x,basis.b=basis.b,CV=CV,...)
+ #             print(2)
+
+
+              a[[i]]<-fregre.gsam(formula,data=newdata,family=family, 
+                                  weights=weights,basis.x=basis.x,basis.b=basis.b,CV=CV,...)
+#print(3)              
               prob.group[,i]<-a[[i]]$fitted.values
             }
-   yest<-ny[apply(prob.group,1,which.min)]######no sera which.max
-   yest<-factor(yest,levels=ny)
+   yest<-lev[apply(prob.group,1,which.min)]######no sera which.max
+   yest<-factor(yest,levels=lev)
    tab<-table(yest,y)
    for (i in 1:ngroup) {     prob[i]=tab[i,i]/sum(tab[,i])     }
-   names(prob)<-ny
+   names(prob)<-lev
 }
 max.prob=sum(diag(tab))/sum(tab)
 output<-list(fdataobj=fdataobj,group=y,group.est=as.factor(yest),
-prob.classification=prob,prob.group=prob.group,C=C,m=m,max.prob=max.prob,fit=a,
+prob.classification=prob,prob.group=prob.group,C=C,m=m,max.prob=max.prob,
 formula=formula,data=newdata)
+output$fit <- a
 class(output)="classif"
 return(output)
 }
 
+
+#############################
+classif.glm2boost=function(group,fdataobj,family=binomial(),basis.x=NULL,
+                           basis.b=NULL,CV=FALSE,...){
+  C<-match.call()
+  a<-list()
+  mf <- match.call(expand.dots = FALSE)
+  m <- match(c("group","fdataobj","family","basis.x","basis.b","CV"), names(mf), 0L)
+  if (is.fdata(fdataobj))   {
+    dataf<-data.frame("y"=group)
+    ldata<-list("df"=dataf,"X"=fdataobj)
+    formula<-formula(y~X)       
+  }
+  else {
+    if (is.matrix(fdataobj)) nms<-colnames(fdataobj)
+    else nms<-names(fdataobj)        
+    #quita [ o $ del names
+    #for (i in 1:length(nms)) nms[i]<-unlist(strsplit(nms[i], "[$]"))[[1]]
+    #for (i in 1:length(nms)) nms[i]<-unlist(strsplit(nms[i], "[[]"))[[1]]
+    dataf<-data.frame("y"=group,fdataobj)
+    ldata<-list("df"=dataf)
+    aaa<-paste(nms,collapse="+")
+    formula<-formula(paste("y~",aaa,sep=""))      
+  }
+  newy<-y<-ldata$df$y
+  if (!is.factor(y)) y<-as.factor(y)
+  n<-length(y)
+  newdata<-ldata
+  ngroup<-nlevels(y)
+  lev<-levels(y)
+  prob<-rep(NA,ngroup)
+  if (!is.null(basis.x)) basis.x=list("X"=basis.x)
+  if (!is.null(basis.b)) basis.b=list("X"=basis.b)  
+  #nlevels(y) cambiar todos
+  a<-list()
+  if (ngroup==2) {
+    #      lev<-as.numeric(names(table(y)))
+    newy<-ifelse(y==lev[1],0,1)
+    newdata$df$y<-newy
+    a[[1]]<-fregre.glm(formula,data=newdata,family=binomial,basis.x=basis.x,
+                       basis.b=basis.b,CV=CV)                
+    yest<-ifelse(a[[1]]$fitted.values<.5,lev[1],lev[2])
+    tab<-table(yest,y)
+    prob[1]=tab[1,1]/sum(tab[,1])
+    dtab<-dim(tab)
+    if (dtab[1]==dtab[2])    {
+      prob[2]=tab[2,2]/sum(tab[,2])
+      names(prob)<-lev
+    }
+    else prob[2]<-0
+    prob.group<- a[[1]]$fitted.values
+    yest<-factor(yest,levels=lev)
+  }
+  else {
+    #   lev<-as.numeric(names(table(y)))
+    prob.group<-array(NA,dim=c(n,ngroup))
+    colnames(prob.group)<-lev
+    for (i in 1:ngroup) {
+      newy<-ifelse(y==lev[i],0,1)
+      newdata$df$y<-newy
+      a[[i]]<-fregre.glm(formula,data=newdata,family=family,basis.x=basis.x,
+                         basis.b=basis.b,CV=CV)
+      prob.group[,i]<-a[[i]]$fitted.values
+    }
+    yest<-lev[apply(prob.group,1,which.min)]
+    yest<-factor(yest,levels=lev)
+    tab<-table(yest,y)
+    for (ii in 1:ngroup) {prob[ii]=tab[ii,ii]/sum(tab[,ii])}
+    names(prob)<-lev
+  }
+  max.prob=sum(diag(tab))/sum(tab)
+  output<-list(fdataobj=fdataobj,group=y,group.est=as.factor(yest),
+               prob.classification=prob,prob.group=prob.group,C=C,m=m,max.prob=max.prob
+               ,formula=formula,data=newdata)
+  output$fit<-a  
+  class(output)="classif"
+  return(output)
+}
 #######################
-classif.tree2boost=function(group,fdataobj,basis.x=NULL,basis.b=NULL,...){   
+classif.rpart2boost=function(group,fdataobj,basis.x=NULL,basis.b=NULL,...){   
 if (!is.factor(group)) group<-as.factor(group)
    C<-match.call()
    mf <- match.call(expand.dots = FALSE)
@@ -202,7 +209,7 @@ if (!is.factor(group)) group<-as.factor(group)
 #   dataf<-list("y"=group,"X"=X)
    formula<-formula(y~X)     
 #   basis.x<-list("X"=basis.x2)
-   fit<-classif.tree(formula,data=ldata,basis.x=basis.x,basis.b=basis.b,...)       
+   fit<-classif.rpart(formula,data=ldata,basis.x=basis.x,...)       
   output<-list(fit=fit,formula=formula,fdataobj=dataf,basis.x=basis.x,
   basis.b=basis.b,group=group,C=C,max.prob=fit$max.prob,"prob.group"=fit$prob.group,
   "prob.classification"=fit$prob.classification,"group.est"=fit$group.est)
@@ -230,7 +237,7 @@ m <- match(c("group","fdataobj","family","weights","par.metric","par.np","offset
    newdata<-ldata
    ngroup<-nlevels(y)
    prob<-rep(NA,ngroup)
-   ny<-levels(y)
+   lev<-levels(y)
 #newdata<-data
 if (!is.null(par.np)) {
    par.np=list("X"=par.np)
@@ -240,40 +247,40 @@ if (!is.null(par.np)) {
 else          par.np =list("X"=list(Ker=AKer.norm,type.S="S.NW"))
 
 if (ngroup==2) {
-      newy<-ifelse(y==ny[1],0,1)
+      newy<-ifelse(y==lev[1],0,1)
       newdata$df$y<-newy
       a[[1]]<-fregre.gkam(formula,data=newdata,family=family,weights=weights,
       par.metric=par.metric,par.np=par.np,offset=offset,control=control,...)
-      yest<-ifelse(a[[1]]$fitted.values<.5,ny[1],ny[2])
+      yest<-ifelse(a[[1]]$fitted.values<.5,lev[1],lev[2])
             tab<-table(yest,y)
       prob[1]=tab[1,1]/sum(tab[,1])
       dtab<-dim(tab)
       if (dtab[1]==dtab[2])    {
            prob[2]=tab[2,2]/sum(tab[,2])
-           names(prob)<-ny
+           names(prob)<-lev
          }
       else prob[2]<-0
       prob.group<-a$fitted.values
-      yest<-factor(yest,levels=ny)
+      yest<-factor(yest,levels=lev)
    }
 else {
-#   ny<-levels(y)
+#   lev<-levels(y)
    prob.group<-array(NA,dim=c(n,ngroup))
-   colnames(prob.group)<-ny
+   colnames(prob.group)<-lev
    for (i in 1:ngroup) {
-              newy<-ifelse(y==ny[i],0,1)
+              newy<-ifelse(y==lev[i],0,1)
               newdata$df$y<-newy
               a[[i]]<-fregre.gkam(formula,data=newdata,family=family,weights=weights,
               par.metric=par.metric,par.np=par.np,offset=offset,control=control,...)
               prob.group[,i]<-a[[i]]$fitted.values
               }
-   yest<-ny[apply(prob.group,1,which.min)]#no sera which.max
-   yest<-factor(yest,levels=ny)
+   yest<-lev[apply(prob.group,1,which.min)]#no sera which.max
+   yest<-factor(yest,levels=lev)
    tab<-table(yest,y)
    for (ii in 1:ngroup) {
        prob[ii]=tab[ii,ii]/sum(tab[,ii])
        }
-  names(prob)<-ny
+  names(prob)<-lev
 }
 max.prob=sum(diag(tab))/sum(tab)
 output<-list(fdataobj=fdataobj,group=y,group.est=yest,
@@ -301,25 +308,25 @@ m <- match(c( "formula","data","w","family","basis.x","basis.b","CV"), names(mf)
 newy<-y<-data$df[[response]]
 if (!is.factor(y)) y<-as.factor(y)
 n<-length(y)
-ny<-levels(y)
+lev<-levels(y)
 newdata<-data
 prob2<-prob<-ngroup<-length(table(y))
 if (ngroup==2) {
-      ny<-as.numeric(names(table(y)))
-      newy<-ifelse(y==ny[1],0,1)
+      lev<-as.numeric(names(table(y)))
+      newy<-ifelse(y==lev[1],0,1)
       newdata$df$y<-newy
       a[[1]]<-fregre.glm(formula,data,family=family,basis.x=basis.x,basis.b=basis.b,CV=CV,...)
 
       if (CV) prediction<-a[[1]]$pred.cv
       else prediction<-a[[1]]$fitted.values
       yest<-ifelse(prediction<.5,0,1)
-      yest<-factor(yest,levels=ny)
+      yest<-factor(yest,levels=lev)
       tab<-table(yest,y)
    prob[1]=tab[1,1]/sum(tab[,1])
       dtab<-dim(tab)
       if (dtab[1]==dtab[2])    {
            prob[2]=tab[2,2]/sum(tab[,2])
-           names(prob)<-ny
+           names(prob)<-lev
          }
       else prob[2]<-0
       prob.group<-prediction
@@ -327,21 +334,21 @@ if (ngroup==2) {
       #devolver a mayores y estimada!
    }
 else {
-   ny<-as.numeric(names(table(y)))
+   lev<-as.numeric(names(table(y)))
    prob.group<-array(NA,dim=c(n,ngroup))
-   colnames(prob.group)<-ny
+   colnames(prob.group)<-lev
    for (i in 1:ngroup) {
-              newy<-ifelse(y==ny[i],0,1)
+              newy<-ifelse(y==lev[i],0,1)
               newdata$df$y<-newy
               a[[i]]<-fregre.glm(formula,data,family=family,basis.x=basis.x,basis.b=basis.b,CV=CV,...)
       if (CV) prediction<-a[[1]]$pred.cv
       else prediction<-a[[1]]$fitted.values
             }
-   yest<-ny[apply(prob.group,1,which.min)]
-   yest<-factor(yest,levels=ny)
+   yest<-lev[apply(prob.group,1,which.min)]
+   yest<-factor(yest,levels=lev)
    tab<-table(yest,y)
    for (i in 1:ngroup) { prob[i]=tab[i,i]/sum(tab[,i])     }
-           names(prob)<-ny
+           names(prob)<-lev
 }
 lev<-levels(y)
 max.prob=sum(diag(tab))/sum(tab)
@@ -349,4 +356,4 @@ output<-list(formula=formula,data=data,group=y,group.est=factor(yest,levels=lev)
 prob.classification=prob,prob.group=prob.group,C=C,fit=a,m=m,max.prob=max.prob)
 class(output)="classif"
 return(output)
-}
\ No newline at end of file
+}
